<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: OperationInterval.php 4251 2006-02-16 16:36:02Z andrew@m3.net $
*/

define('SECONDS_PER_MINUTE', 60);
define('MINUTES_PER_HOUR', 60);

/**
 * @package    MaxMaintenance
 * @author     James Floyd <james@m3.net>
 */
class OperationInterval
{

    var $interval;

   /**
    * Singleton interface to return a new OperationInterval
    * object
    * @return OperationInterval
    */
    function &singleton()
    {
        static $instance;
        if (!is_a($instance, 'OperationInterval')) {
            $instance = new OperationInterval();
        }
        return $instance;
    }

   /**
    * Constructor - populates class property $this->interval
    * with the values set in configuration file
    */
    function operationInterval()
    {
        $this->interval = $GLOBALS['_MAX']['CONF']['maintenance']['operationInterval'];
    }

   /**
    * Method to return a PEAR::Date object representing a given date
    * or the current date and time if no date is supplied
    *
    * @param string $date date time in format YYY-MM-DD HH:MM:SS
    * @return objects of type PEAR::Date
    * @access private
    */
    function _getDate($date = '')
    {
        return (empty($date)) ? new Date() : new Date($date);
    }

   /**
    * A class to test if an operation interval value is valid.
    *
    * @param integer $oi The operation interval value in minutes.
    * @return mixed True if the operation interval value is valid, a {@link PEAR_Error}
    *              object with error type MAX_ERROR_INVALIDOPERATIONINT otherwise.
    */
    function isValid()
    {
        if ($this->interval < 1) {
            // Operation interval must be at least every minute
            $error = 'The operation interval of ' . $this->interval . ' is invalid';
            return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
        } elseif ($this->interval < 60) {
            // Operation interval is more frequent than once an hour
            if ((60 % $this->interval) != 0) {
                // Operation interval must be a factor of 60 minutes
                $error = 'The operation interval of ' . $this->interval . ' is invalid';
                return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
            }
        } elseif ($this->interval > 60) {
            // Operation interval is more often than once an hour
            if (($this->interval % 60) != 0) {
                // Operation interval must be a multiple of 60 minutes
                $error = 'The operation interval of ' . $this->interval . ' is invalid';
                return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
            }
            if ($this->interval > MINUTES_PER_DAY) {
                // Operation interval must not be more than one week (24 * 60 = 1440 minutes)
                $error = 'The operation interval of ' . $this->interval . ' is invalid';
                return MAX::raiseError($error, MAX_ERROR_INVALIDOPERATIONINT);
            }
        }
        return true;
    }

   /**
    * Method to return the number of operation intervals in a given
    * date range.  Range must be valid operation interval start & end date
    *
    * @param object $oStartDate PEAR::Date object
    * @param object $oEndDate PEAR::Date object
    * @return integer number of operation intervals remaining
    */
    function getRemaining($oStartDate, $oEndDate)
    {
        // Check $oStartDate, $oEndDate are PEAR::Date objects
        if (!(is_a($oStartDate, 'Date')) || !(is_a($oEndDate, 'Date'))) {
            return MAX::raiseError('Invalid parameter pass to ' . __CLASS__ . '::' . __FUNCTION__ . '. Not a PEAR::Date object', MAX_ERROR_INVALIDARGS);
        }
        // Get timestamp of start date/time - in seconds
        $startSeconds = mktime(
            $oStartDate->getHour(), $oStartDate->getMinute(),
            $oStartDate->getSecond(), $oStartDate->getMonth(),
            $oStartDate->getDay(), $oStartDate->getYear()
        );
        // Get timestamp of end date/time - in seconds
        $endSeconds = mktime(
            $oEndDate->getHour(), $oEndDate->getMinute(),
            $oEndDate->getSecond(), $oEndDate->getMonth(),
            $oEndDate->getDay(), $oEndDate->getYear()
        );
        // Calculate interval length in seconds
        $intervalSeconds = $endSeconds - $startSeconds;
        // Find number of operation intervals during interval
        return ($intervalSeconds <= 0) ? 0 : round($intervalSeconds / $this->secondsPerInterval());
    }

   /**
    * Method to return a single operation interval's start and end
    * date time limits for a given PEAR::Date object
    *
    * @param    object $date of type PEAR::Date
    *
    * @return   array of PEAR::Date objects representing start & end of single
    *           operation interval
    *
    * @access   public
    */
    function convertDateToOperationIntervalStartAndEndDates($oDate)
    {
        // Get the date representing the start of the week
        $oStartOfWeek = $this->_getDate(Date_Calc::beginOfWeek($oDate->getDay(),
            $oDate->getMonth(), $oDate->getYear(), '%Y-%m-%d 00:00:00'));
        // Get the operation interval ID of the date
        $intervalID = $this->getIntervalID($oDate);
        // The start of the operation interval is the start of the week plus the
        // operation interval ID multiplied by the operation interval
        $oStart = new Date();
        $oStart->copy($oStartOfWeek);
        $oStart->addSeconds($intervalID * $this->secondsPerInterval());
        // The end of the operation interval is the start of the week plus
        // one operation interval  ID + 1 multiplied by the operation interval
        $oEnd = new Date();
        $oEnd->copy($oStart);
        $oEnd->addSeconds($this->secondsPerInterval() - 1);
        // Return the result
        return array('start' => $oStart, 'end' => $oEnd);
    }

   /**
    * Method to return a single operation interval's start and end
    * date time limits for the operation interval before the one that
    * contains a given PEAR::Date object.
    *
    * @access   public
    * @param    object $date of type PEAR::Date.
    * @return   array of PEAR::Date objects representing start & end of single
    *           operation interval.
    */
    function convertDateToPreviousOperationIntervalStartAndEndDates($oDate)
    {
        // Get the start & end dates of the operation interval that
        // contains the current date
        $aResult = $this->convertDateToOperationIntervalStartAndEndDates($oDate);
        // Subtract one second from the start date
        $oNewDate = new Date();
        $oNewDate->copy($aResult['start']);
        $oNewDate->subtractSeconds(1);
        // Return the result from the new date
        return $this->convertDateToOperationIntervalStartAndEndDates($oNewDate);
    }

   /**
    * Method to return a single operation interval's start and end
    * date time limits for the operation interval after the one that
    * contains a given PEAR::Date object.
    *
    * @access   public
    * @param    object $date of type PEAR::Date.
    * @return   array of PEAR::Date objects representing start & end of single
    *           operation interval.
    */
    function convertDateToNextOperationIntervalStartAndEndDates($oDate)
    {
        // Get the start & end dates of the operation interval that
        // contains the current date
        $aResult = $this->convertDateToOperationIntervalStartAndEndDates($oDate);
        // Add one second to the end date
        $oNewDate = new Date();
        $oNewDate->copy($aResult['end']);
        $oNewDate->addSeconds(1);
        // Return the result from the new date
        return $this->convertDateToOperationIntervalStartAndEndDates($oNewDate);
    }

   /**
    * Method to return an array of start & end Date objects
    * representing a range of operation intervals, return array is
    * keyed by intervalID as a result only the last date for a given
    * intervalID will be returned
    *
    * This method replaced Maintenance::getOperationIntervalRange()
    *
    * @param object     $date PEAR::Date object representing the first operation interval
    *                   of the range to be calculated.
    *
    * @param integer    $intervals The number of operation intervals in the range,
    *                   inclusive of the first and last intervals.
    *
    * @return array     keyed by the operation interval ID containing two PEAR::Date objects
    *                   representing the start and end of the interval
    */
    function getIntervalIdRange($oDate, $intervals)
    {
        // Calculate the start and end date of the first interval
        $aLimits = $this->convertDateToOperationIntervalStartAndEndDates($oDate);

        // Calculate the start interval ID
        $intervalID = $this->getIntervalID($aLimits['start']);
        // Store first interval start & end dates
        $operationIntervals[$intervalID] = $aLimits;

        for ($counter = 1; $counter < $intervals; $counter++) {
            // Calculate a date in the next operation interval
            $oNextDate = $this->_getDate();
            $oNextDate->copy($operationIntervals[$intervalID]['end']);
            $oNextDate->addSeconds(1);

            // Calculate the start and end date of the next operation interval
            $aLimits = $this->convertDateToOperationIntervalStartAndEndDates($oNextDate);

            // Calculate the interval ID
            $intervalID = $this->getIntervalID($aLimits['start']);
            // Store the operation interval dates
            $operationIntervals[$intervalID] = $aLimits;
        }
        return $operationIntervals;
    }

    /**
     * A method to return an array of PEAR::Date objects, with one object
     * for each day of a date range, given start and end dates.
     *
     * @param PEAR::Date $oStartDate The PEAR::Date object representing the start operation
     * 								 interval of the range to be calculated.
     * @param PEAR::Date $oEndDate The PEAR::Date object representing the end operation
     * 							   interval of the range to be calculated.
     * @return array An array of PEAR::Date objects, each set to the start second (e.g.
     * 			     "2005-12-09 00:00:00") of the day, with one object per day in the
     * 				 date range specified by the parameters.
     */
    function getDayRange($oStartDate, $oEndDate)
    {
    	// Prepare the result array
    	$aDays = array();
        // Get span object
        $span = new Date_Span();
        $oStartDateCopy = new Date();
        $oStartDateCopy->copy($oStartDate);
        $oEndDateCopy = new Date();
        $oEndDateCopy->copy($oEndDate);
        $span->setFromDateDiff($oStartDateCopy, $oEndDateCopy);
        // Get the number of days - if any hours, minutes or seconds, add 1 day
        $days = (($span->hour > 0) || ($span->minute > 0) || ($span->second > 0))
                ? ($span->day + 1) : $span->day;
        // Create a date at the start of the opening day
        $oDate = new Date();
        $oDate->copy($oStartDate);
        $oDate->setHour(0);
        $oDate->setMinute(0);
        $oDate->setSecond(0);
        // Loop over the required number of days
        for ($i = 0; $i < $days; $i++) {
        	// Store the date
        	$oStoreDate = new Date();
        	$oStoreDate->copy($oDate);
            $aDays[] = array($oStoreDate);
            // Go to the next day
            $oDate->addSeconds(SECONDS_PER_DAY);
        }
        // Return the result
        return $aDays;
    }

   /**
    * A method to convert a Date object into the appropriate operation
    * interval ID (of the week).
    *
    * Works by finding out how many days, hours, minutes into the week the time is, and
    * then converting that into how many minutes into the week the time is. This value
    * is then divided by the interval length (in minutes) to determine how many intervals
    * into the week the time is.
    *
    * This method replaces Maintenance::convertDateToOperationIntervalID()
    *
    * @param PEAR::Date $oDate PEAR::Date object
    * @return integer The operation interval ID that the date is in.
    */
    function getIntervalID($oDate)
    {
        // Calculate minutes elapsed since start of week
        $totalMinutes = ($oDate->getDayOfWeek() * MINUTES_PER_DAY)
            + ($oDate->getHour() * MINUTES_PER_HOUR) + $oDate->getMinute();
        return(floor($totalMinutes / $this->interval));
    }

    function getNextIntervalID($intervalId)
    {
        // interval id representing end of week
        $endIntervalId = ((MINUTES_PER_WEEK / $this->interval) - 1);

        if ($intervalId >= ($endIntervalId)) {
            if ($intervalId == ($endIntervalId)) {
                return 0;
            } else {
                return MAX::raiseError('Value for $intervalId can not be > intervals in a week',
                    MAX_ERROR_INVALIDARGS, PEAR_ERROR_DIE);
            }
        } else {
            return ($intervalId + 1);
        }
    }

   /**
    * A static method to find the previous operation interval ID.
    *
    * @param integer $operationIntervalID The operation interval ID.
    * @return integer The previous operation interval ID.
    */
    function getPreviousIntervalID($intervalID)
    {
        if ($intervalID == 0) {
            return ((MINUTES_PER_WEEK / $this->interval) - 1);
        } else {
            return ($intervalID - 1);
        }
    }

   /**
    * Method to increment an interval ID by given number of intervals
    *
    * @param integer $intervalId The operation interval ID.
    * @param integer $intervals number of intevals to increment
    * @return integer Resulting operation interval ID
    */
    function incrementIntervalID($intervalId, $intervals)
    {
        if ($intervals < 0) {
            return MAX::raiseError('Value for $intervalId can not be less than zero',
                MAX_ERROR_INVALIDARGS, PEAR_ERROR_DIE);
        }

        for ($i = 0; $i < $intervals; $i++) {
            $intervalId = $this->getNextIntervalID($intervalId);
        }
        return $intervalId;
    }

   /**
    * Method to decrement an interval id by given number of intervals
    *
    * @param integer $intervalId the current operation interval id
    * @param integer $intervals number of intevals to decrement
    * @return integer interval id
    */
    function decrementIntervalID($intervalId, $intervals)
    {
        if ($intervals < 0) {
            return MAX::raiseError('Value for $intervalId can not be less than zero',
                MAX_ERROR_INVALIDARGS, PEAR_ERROR_DIE);
        }

        for ($i = 0; $i < $intervals; $i++) {
            $intervalId = $this->getPreviousIntervalID($intervalId);
        }
        return $intervalId;
    }

   /**
    * A method to return the next Operation Interval(OI) (** not current OI **)
    * returns an array of PEAR::Date object representing
    * the start array['start'] and end array['end'] of the next operation interval
    *
    * @param string $date date time in format YYY-MM-DD HH:MM:SS
    *
    * @return array containing start and end date same as response from
    *           {@link MAX_Maintenance_OperationInterval::convertDateToOperationIntervalStartAndEndDates()}
    *
    * @todo should param be a pear date object?
    */
    function getNext($date = '') {
        $oDate = $this->_getDate($date);
        $oDate->addSeconds($this->secondsPerInterval());
        return $this->convertDateToOperationIntervalStartAndEndDates($oDate);
    }

    function getPrevious($date = '') {
        $oDate = $this->_getDate($date);
        $oDate->subtractSeconds($this->secondsPerInterval());
        return $this->convertDateToPreviousOperationIntervalStartAndEndDates($oDate);
    }

   /**
    * Method to return the number of operation intervals
    * during a 24 hour period
    *
    * @return integer number of operation intervals
    */
    function intervalsPerDay()
    {
        return (SECONDS_PER_DAY / $this->secondsPerInterval());
    }

    function intervalsPerWeek()
    {
        return (7 * $this->intervalsPerDay());
    }

   /**
    * Method to return the number of seconds per operation interval
    *
    * @return integer number of seconds per operation interval
    */
    function secondsPerInterval()
    {
        return $this->interval * SECONDS_PER_MINUTE;
    }

   /**
    * Method to return the number of operation intervals
    * remaining within a given day.  Date object MUST
    * represent the start of a valid operation interval
    *
    * @param $oDate PEAR::Date object
    * @return integer number of intervals remaining
    */
    function getRemainingByDate($oDate)
    {
        // if not start of the day
        if (($oDate->getHour() != 0) || ($oDate->getMinute() != 0) || ($oDate->getSecond() != 0)) {
            $oEnd = new Date();
            $oEnd->copy($oDate);
            $oEnd->setHour(23);
            $oEnd->setMinute(59);
            $oEnd->setSecond(59);
            return $this->getRemaining($oDate, $oEnd);
        }
        return $this->intervalsPerDay();
    }

   /**
    * Method to return the operation interval represented by a given
    * PEAR::Date object in a 24 hour period.  Returns value between
    * 0 - 23 for a 60 minute interval.
    *
    * Example: An operation interval of 60 minutes would
    * give a total of 24 operation intervals per day.  If a date object
    * is tested with the value 2005-05-02 00:00:00 (start of day)
    * this method would return 0 as this is the first OI in a day.
    *
    * For a date value of 2005-05-02 05:00:00 integer 5 will be returned
    * For a date value of 2005-05-02 23:00:00 integer 23 will be returned
    *
    * @param object $oDate
    * @return integer
    */
    function getDayIntervalNumber($oDate)
    {
        return ($this->intervalsPerDay() - $this->getRemainingByDate($oDate));
    }

}

?>
